#include <unistd.h>

#include <QFileDialog>
#include <QFontDialog>
#include <QFile>
#include <QTextStream>
#include <QIcon>
#include <QNetworkRequest>
#include <QNetworkReply>
#include <QNetworkProxyFactory>
#include <QNetworkAccessManager>
#include <QtScript>
#include <QtSql>
#include <QInputDialog>
#include <QMessageBox>

#include <QDebug>

#include "cookiesaver.h"
#include "password.h"
#include "addplaylistitem.h"
#include "addurl.h"
#include "utils.h"
#include "settingsui.h"
#include "proxyfactory.h"

SettingsDialog::SettingsDialog(QWidget *parent)
    : QDialog(parent)
{
    ui.setupUi(this);

    settings = Settings::load();
    m_testProcess = 0;

    QIcon::setThemeName(Utils::getIconTheme());
    finalizeUi();
    applySettings();
}

void SettingsDialog::finalizeUi()
{
    // FIXME: don't hardcode /share/ path
    ui.headerLabel->setPixmap(QPixmap("/usr/share/pixmaps/linutop-webkiosk.png"));

    ui.startupChk->setChecked(settings->autostarted());
    ui.passwordBtn->setEnabled(false);
    ui.restrictionDeleteBtn->setEnabled(false);

    QItemSelectionModel *model = ui.restrictionsListView->selectionModel();
    connect(model, SIGNAL(selectionChanged(const QItemSelection &, const QItemSelection &)),
            this, SLOT(on_restrictionsSelection_changed(const QItemSelection &, const QItemSelection &)));

    m_localPlaylistView = new TreeView(this);
    ui.playlistLayout->insertWidget(0, m_localPlaylistView, 1);
    connect(m_localPlaylistView, SIGNAL(dataChanged()),
            this, SLOT(genPlaylistList()));

    ui.localPlaylistDeleteBtn->setEnabled(false);
    ui.localPlaylistUpBtn->setEnabled(false);
    ui.localPlaylistDownBtn->setEnabled(false);

    // icons for buttons
    ui.restrictionAddBtn->setIcon(QIcon::fromTheme("list-add"));
    ui.restrictionDeleteBtn->setIcon(QIcon::fromTheme("list-remove"));
    ui.restrictionImportBtn->setIcon(QIcon::fromTheme("document-import"));
    ui.restrictionExportBtn->setIcon(QIcon::fromTheme("document-export"));

    ui.localPlaylistAddBtn->setIcon(QIcon::fromTheme("list-add"));
    ui.localPlaylistDeleteBtn->setIcon(QIcon::fromTheme("list-remove"));
    ui.localPlaylistUpBtn->setIcon(QIcon::fromTheme("go-up"));
    ui.localPlaylistDownBtn->setIcon(QIcon::fromTheme("go-down"));
    ui.localPlaylistImportBtn->setIcon(QIcon::fromTheme("document-import"));
    ui.localPlaylistExportBtn->setIcon(QIcon::fromTheme("document-export"));
    ui.localPlaylistCookiesBtn->setIcon(QIcon::fromTheme("emblem-favorite"));

    ui.okBtn->setIcon(QIcon::fromTheme("gtk-ok"));
    ui.cancelBtn->setIcon(QIcon::fromTheme("gtk-cancel"));
    ui.helpBtn->setIcon(QIcon::fromTheme("gtk-help"));
    ui.testBtn->setIcon(QIcon::fromTheme("stock_media-play"));

    QStringList dataArray;
    dataArray << "false"<<"1;d"<<"3;d"<<"7;d"<<"21;d"<<"1;m"<<"3;m";
    foreach (QString data, dataArray) {
        ui.cleanAfter->setItemData(dataArray.indexOf(data),data);
    }
#ifdef DEMO
    ui.homeEdit->setEnabled(false);
    ui.label_2->setText(tr("<html><head/><body><p><span style='font-size:12pt; font-weight:600;'>Linutop Kiosk - Setup - Demo for Raspberry Pi</span><span style='font-size:12pt;'><br/></span><span style='font-style:italic;'>For Secure Internet Acces and Digital Signage</span></p></body></html>"));
#endif

    // temporary
    ui.localPlaylistImportBtn->setVisible(false);
    ui.localPlaylistExportBtn->setVisible(false);
    ui.playlistUsbUpdate->setVisible(false);
    ui.cleanLabel->setVisible(false);
    ui.cleanAfter->setVisible(false);


}

void SettingsDialog::setText(QLineEdit *obj, const char *path)
{
    obj->setText(settings->get(path).toString());
}

void SettingsDialog::setChecked(QCheckBox *obj, const char *path, bool value)
{
    obj->setChecked(settings->get(path).toBool() == value);
}

void SettingsDialog::setSpin(QSpinBox *obj, const char *path)
{
    obj->setValue(settings->get(path).toInt());
}

void SettingsDialog::applySettings()
{
    QTabBar *bar = ui.tabWidget->tabBar();

    if (settings->get("main/mode") == "kiosk")
    {
        if (settings->get("main/after_web_session") == "session")
        {
            ui.useKioskRadio->setChecked(true);
            bar->setTabEnabled(1, true);
            bar->setTabEnabled(2, false);
        }
        else
        {
            ui.useBothChk->setChecked(true);
            bar->setTabEnabled(1, true);
            bar->setTabEnabled(2, true);
        }
    }
    else
    {
        ui.useDisplayChk->setChecked(true);
        bar->setTabEnabled(1, false);
        bar->setTabEnabled(2, true);
    }

    setChecked(ui.fullscreenChk, "main/fullscreen", true);
    setChecked(ui.screenSaverChk, "main/no_screensaver", true);

    if (! settings->get("main/admin_passwd").toString().isEmpty())
        ui.passwordChk->setChecked(true);

    QString fontStr(settings->get("webpage/default_font").toString());
    QFont font(fontStr);
    ui.fontBtn->setText(QString("%1 (%2)").arg(font.family()).arg(font.pointSize()));

    // webkiosk
    setChecked(ui.hideToolbarChk, "main/hide_toolbar", true);
    setChecked(ui.browseFSChk, "main/can_access_fs", true);
    setChecked(ui.displayClearChk, "main/can_end_session", true);
    setChecked(ui.printChk, "main/can_print", true);
    setChecked(ui.sessionAtStartupChk, "main/start_with_session", true);
    setText(ui.homeEdit, "main/default_url");
    setSpin(ui.sessionTimeoutSpin, "main/stop_session_after_seconds");

    setChecked(ui.sessionTimeChk, "main/with_timer", true);
    setSpin(ui.sessionTimeSpin, "main/default_timer");
    setText(ui.postSessionUrlEdit, "main/post_session_uri");
    setSpin(ui.postSessionTimeSpin, "main/post_session_uri_time");

    QString lmode = settings->get("main/list_mode").toString();
    int lindex = 0;
    if (lmode == "whitelist")
        lindex = 2;
    else if (lmode == "blacklist")
        lindex = 1;
    ui.restrictionsCombo->setCurrentIndex(lindex);
    setChecked(ui.regexChk, "main/list_use_regex", true);
    QString f(settings->regexFile());
    restrictionsLoadFromFile(f);
    genRestrictionsList();

    // display
    setChecked(ui.resizeChk, "slideshow/resize", true);
    setChecked(ui.hideMouseChk, "slideshow/hide_pointer", true);
    setChecked(ui.muteChk, "slideshow/mute", true);
    setSpin(ui.displayTimeSpin, "slideshow/default_time");

    if(settings->get("slideshow/use_local").toBool()){
        ui.updateLocalRadio->setChecked(true);
        loadChannels();
    }
    else if (settings->get("slideshow/use_remote").toBool()){
        ui.updateOnlineRadio->setChecked(true);
        loadChannels();
    }
    else
        ui.updatePlaylistRadio->setChecked(true);
    setChecked(ui.remoteChk,"remote/enabled",true);
    ui.directChk->setEnabled(ui.remoteChk->isChecked());
    setChecked(ui.directChk,"slideshow/direct",true);
    setText(ui.remoteEdit, "slideshow/url");
    ui.remoteEdit->setEnabled(ui.updateLocalRadio->isChecked());
    ui.stackedWidget_7->setCurrentIndex(ui.updatePlaylistRadio->isChecked());
    setSpin(ui.refreshPlaylistSpin, "slideshow/refresh_after_minutes");
    setChecked(ui.playlistUsbUpdate, "slideshow/do_local_update", true);
    playlistLoadFromFile();
    genPlaylistList();
    connect(m_localPlaylistView->selectionModel(),
            SIGNAL(selectionChanged(const QItemSelection &, const QItemSelection &)),
            this,
            SLOT(on_PlaylistSelectionChanged(const QItemSelection &, const QItemSelection &)));

    // network
    switch (settings->get("network/proxy_type").toInt())
    {
        case 1:
            ui.proxyAutoRadio->setChecked(true);
            break;
        case 2:
            ui.proxyManualRadio->setChecked(true);
            break;
        default:
            ui.proxyNoRadio->setChecked(true);
            break;
    }

    setText(ui.proxyUrlEdit, "network/proxy_uri");
    setText(ui.proxyUserEdit, "network/proxy_user");
    setText(ui.proxyPasswordEdit, "network/proxy_passwd");
    setText(ui.invoice,"slideshow/invoice");
    setText(ui.mail,"slideshow/email");
    setText(ui.newTabUrl,"main/new_tab_url");

    ui.channelLabel->setText(settings->get("slideshow/channel").toString());
}

void SettingsDialog::playlistLoadFromFile()
{
    QFile f(settings->playlistFile());
    if (! f.open(QIODevice::ReadOnly | QIODevice::Text))
        return;

    QTextStream in(&f);
    while (!in.atEnd())
    {
        QString line = in.readLine();
        QRegExp rx("^([^#].*)\\s+(\\d+)\\s*$");
        rx.setMinimal(true);
        if (rx.exactMatch(line))
        {
            QStringList list = rx.capturedTexts();
            m_localPlaylistView->add(list.at(1), list.at(2).toInt());
        }
    }
    f.close();
}

QString cleanJsonString(QString data)
{
    // Remove inline comms
    QRegExp pattern = QRegExp("(^|\\[|\\{|,|\\n|\\s)//.*($|\\n)");
    pattern.setMinimal(true); //ungreedy
    data.replace(pattern, "\\1\n");
    data.replace(pattern, "\\1\n"); //2 times, I am not sure why...
    // Remove bloc comms
    pattern = QRegExp("/\\*.*\\*/");
    pattern.setMinimal(true); //ungreedy
    data.replace(pattern, "");
    // Add first and last brace
    if (!data.startsWith("{")) {
        data = "{\n"+data;
    }
    if (!data.endsWith("}")) {
        data += "\n}";
    }
    // Remove commas before } or ]
    pattern = QRegExp(",(\\s*[}\\]])");
    pattern.setMinimal(true); //non-greedy
    data.replace(pattern, "\\1");
    return data;
}

void SettingsDialog::loadChannels()
{
    qDebug()<<"loadChannels";
    ProxyFactory * factory = new ProxyFactory;
    QNetworkProxyFactory::setApplicationProxyFactory(factory);
    //qDebug()<<"after proxy factory";
    ui.listWidget->clear();
    QUrl url;
    QString s_url = settings->get("slideshow/use_local").toBool()?settings->get("slideshow/url").toString():"http://linutop.tv";
    QByteArray postData;
    if(settings->get("slideshow/invoice").toString().isEmpty()||settings->get("slideshow/email").toString().isEmpty())
    {
        url = QUrl(s_url+"/webapp/db/channels");
        qDebug()<<"after set url";
    }
    else{
        url = QUrl(s_url+"/webapp/api/user/"+settings->get("slideshow/invoice").toString());
        postData.append("model=");
        QScriptEngine engine;
        QMap<QString, QString> map;
        map["invoice"] = settings->get("slideshow/invoice").toString();
        map["mail"] = settings->get("slideshow/email").toString();
        QScriptValue v = engine.newObject();
        v.setProperty("invoice",settings->get("slideshow/invoice").toString());
        v.setProperty("email",settings->get("slideshow/email").toString());
        engine.evaluate("function toString() { return JSON.stringify(this) }");
        QScriptValue toString = engine.globalObject().property("toString");
        postData.append(toString.call(v).toString());
        qDebug()<<postData;
    }
        QNetworkRequest req;
        req.setUrl(url);
        req.setHeader(QNetworkRequest::ContentTypeHeader,"application/x-www-form-urlencoded; charset=UTF-8");
        QNetworkCookie cookie;
        cookie.setName("lang");
        cookie.setValue("fr");
        QList<QNetworkCookie> cookies;
        cookies.append(cookie);
        QVariant var;
        var.setValue(cookies);
        req.setHeader(QNetworkRequest::CookieHeader,var);
        QNetworkAccessManager nam;
        QNetworkReply * rep = nam.post(req,postData);
        QEventLoop loop;
        connect(rep,SIGNAL(finished()),&loop,SLOT(quit()));
        loop.exec();
        qDebug()<<"AfterLoop"<<rep->errorString();
        if(rep->error()==QNetworkReply::NoError)
        {
            QScriptEngine engine;
            QString re = QString(rep->readAll());
            QScriptValue result = engine.evaluate("("+re.toUtf8()+")");
            QScriptValueIterator it(result.property("screens").isValid()?result.property("screens"):result);
            it.setValue(result.property("screens").isValid()?result.property("screens"):result);
            while(it.hasNext()){
                it.next();
                QListWidgetItem *item = new QListWidgetItem(it.value().property("label").toString());
                item->setData(32,"/webapp/p/"+it.value().property("login").toString()+"-"+it.value().property("screen").toString()+"/"+(it.value().property("token").isValid()?it.value().property("token").toString():"public"));
                ui.listWidget->addItem(item);
            }
            if(settings->get("slideshow/channel").isNull()){
                on_listWidget_itemClicked(ui.listWidget->item(0));
            }
        }
        else{
            qDebug()<<rep->readAll();
        }

}



void SettingsDialog::genPlaylistList()
{
    QStringList l;
    QStandardItemModel *m = static_cast<QStandardItemModel *>(m_localPlaylistView->model());

    for (int row=0; row < m->rowCount(); ++row)
    {
        QString uri = m->item(row, 0)->data(Qt::DisplayRole).toString();
        int time = m->item(row, 1)->data(Qt::DisplayRole).toInt();
        l << QString("%1 %2").arg(uri).arg(time);
    }

    settings->playlistItems = l;
}

bool SettingsDialog::restrictionsLoadFromFile(QString &filename)
{
    ui.restrictionsListView->clear();
    QFile f(filename);
    if (! f.open(QIODevice::ReadOnly | QIODevice::Text))
        return false;

    QTextStream in(&f);
    while (!in.atEnd())
    {
        QString line = in.readLine();
        if (line.isEmpty())
            continue;
        ui.restrictionsListView->addItem(line);
    }
    f.close();

    return true;
}

bool SettingsDialog::restrictionsSaveToFile(QString &filename)
{
    QFile f(filename);
    if (! f.open(QIODevice::WriteOnly | QIODevice::Text))
        return false;

    QTextStream out(&f);
    for (int i=0; i<ui.restrictionsListView->count(); ++i)
        out << ui.restrictionsListView->item(i)->text() << "\n";
    f.close();

    return true;
}

void SettingsDialog::on_restrictionsSelection_changed(
        const QItemSelection &selected,
        const QItemSelection &deselected)
{
    bool b = ui.restrictionsListView->selectionModel()->hasSelection();
    ui.restrictionDeleteBtn->setEnabled(b);
}

void SettingsDialog::showError(QNetworkReply::NetworkError error)
{
    qDebug()<<error;
}

void SettingsDialog::on_helpBtn_clicked()
{
    QProcess *p = new QProcess(this);
    p->start("yelp /usr/share/gnome/help/linutop-webkiosk/C/webkiosk.xml");
}

void SettingsDialog::on_testFinished(int exitCode, QProcess::ExitStatus exitStatus)
{
    QProcess p;
    p.start("rm -rf /tmp/linutop-kiosk-test");
    p.waitForFinished();
    delete m_testProcess;
    m_testProcess = 0;
}

void SettingsDialog::on_testBtn_clicked()
{
    if (m_testProcess) /* test in progress already */
        return;

    QDir d("/tmp/linutop-kiosk-test");
    d.mkpath("/tmp/linutop-kiosk-test");
    settings->save("/tmp/linutop-kiosk-test", true);
    m_testProcess = new QProcess(this);
    connect(m_testProcess, SIGNAL(finished(int, QProcess::ExitStatus)),
            this, SLOT(on_testFinished(int, QProcess::ExitStatus)));
    QStringList args;
    args << "/tmp/linutop-kiosk-test";
    m_testProcess->start("linutop-kiosk", args);
}

void SettingsDialog::on_cancelBtn_clicked()
{
    close();
}

void SettingsDialog::on_okBtn_clicked()
{
    bool canClose = true;
    if (! ui.passwordChk->isChecked())
        settings->set("main/admin_passwd", QString(""));
    if(ui.updateOnlineRadio->isChecked()&&settings->get("slideshow/pages").toString().isEmpty()){
        qDebug()<<QMessageBox::warning(this,tr("No channel"),tr("No Channel has been selected, switching back to local playlist"),tr("Close anyway"),tr("Go back"));
        settings->set("slideshow/use_remote",false);
    }
    else if(ui.updatePlaylistRadio->isChecked()&&m_localPlaylistView->isEmpty()){
        canClose = false;
        QMessageBox::warning(this,tr("Empty playlist"),tr("Your local playlist is empty, please add an item"));
    }
    if(!newTemplate.isEmpty()){
        QFile fi(newTemplate);
        QFile::remove("/usr/share/linutop-webkiosk/text.html");
        fi.copy("/usr/share/linutop-webkiosk/text.html");
    }
    if(canClose){
        settings->save();
        close();
    }
}

/* main */

void SettingsDialog::on_useBtnGrp_buttonClicked(QAbstractButton *btn)
{
    QTabBar *tb = ui.tabWidget->tabBar();
    if (btn == ui.useKioskRadio)
    {
        tb->setTabEnabled(1, true);
        tb->setTabEnabled(2, false);
        settings->set("main/mode", QString("kiosk"));
        settings->set("main/after_web_session", QString("session"));
        settings->set("main/start_with_session", true);
    }
    else if (btn == ui.useDisplayChk)
    {
        tb->setTabEnabled(1, false);
        tb->setTabEnabled(2, true);
        settings->set("main/mode", QString("display"));
    }
    else
    {
        tb->setTabEnabled(1, true);
        tb->setTabEnabled(2, true);
        settings->set("main/mode", QString("kiosk"));
        settings->set("main/after_web_session", QString("slideshow"));
        if (settings->get("main/with_timer").toBool())
            settings->set("main/start_with_session", false);
    }
}

void SettingsDialog::on_fullscreenChk_toggled(bool checked)
{
    settings->set("main/fullscreen", checked);
}

void SettingsDialog::on_screenSaverChk_toggled(bool checked)
{
    settings->set("main/no_screensaver", checked);
}

void SettingsDialog::on_startupChk_toggled(bool checked)
{
    settings->setAutostarted(checked);
}

void SettingsDialog::on_passwordChk_toggled(bool checked)
{
    ui.passwordBtn->setEnabled(checked);
}

void randomString(char *s, const int len) {
    static const char alphanum[] =
        "0123456789"
        "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        "abcdefghijklmnopqrstuvwxyz";

    for (int i = 0; i < len; ++i) {
        s[i] = alphanum[rand() % (sizeof(alphanum) - 1)];
    }

    s[len] = 0;
}

void SettingsDialog::on_passwordBtn_clicked()
{
    PasswordDialog d(this);
    if (d.exec() == QDialog::Accepted)
    {
        char salt[10] = "$1$";
        randomString(salt+3, 6);
        char *c = crypt(d.password().toUtf8().data(), salt);
        settings->set("main/admin_passwd", QString(c));
    }
    d.deleteLater();
}

void SettingsDialog::on_fontBtn_clicked()
{
    bool ok;
    QFont font = QFontDialog::getFont(&ok, settings->get("webpage/default_font").toString(), this);
    if (ok)
    {
        settings->set("webpage/default_font", font.toString());
        ui.fontBtn->setText(QString("%1 (%2)").arg(font.family()).arg(font.pointSize()));
    }
}

/* kiosk */
void SettingsDialog::on_hideToolbarChk_toggled(bool checked)
{
    settings->set("main/hide_toolbar", checked);
}

void SettingsDialog::on_browseFSChk_toggled(bool checked)
{
    settings->set("main/can_access_fs", checked);
}

void SettingsDialog::on_displayClearChk_toggled(bool checked)
{
    settings->set("main/can_end_session", checked);
}

void SettingsDialog::on_printChk_toggled(bool checked)
{
    settings->set("main/can_print", checked);
}

void SettingsDialog::on_sessionAtStartupChk_toggled(bool checked)
{
    settings->set("main/start_with_session", checked);
}

void SettingsDialog::on_homeEdit_textChanged(QString value)
{
    settings->set("main/default_url", value);
}

void SettingsDialog::on_sessionTimeoutSpin_valueChanged(int value)
{
    settings->set("main/stop_session_after_seconds", value);
}

void SettingsDialog::on_sessionTimeChk_toggled(bool checked)
{
    settings->set("main/with_timer", checked);
    settings->set("slideshow/stop_on_event", checked);
}

void SettingsDialog::on_sessionTimeSpin_valueChanged(int value)
{
    settings->set("main/default_timer", value);
}

void SettingsDialog::on_postSessionUrlEdit_textEdited(QString value)
{
    settings->set("main/post_session_uri", value);
}

void SettingsDialog::on_postSessionTimeSpin_valueChanged(int value)
{
    settings->set("main/post_session_uri_time", value);
}

void SettingsDialog::on_restrictionsCombo_currentIndexChanged(int value)
{
    QString s;
    switch (value)
    {
        case 0: s = QString ("none"); break;
        case 1: s = QString ("blacklist"); break;
        case 2: s = QString ("whitelist"); break;
    }
    settings->set("main/list_mode", s);
}

void SettingsDialog::on_regexChk_toggled(bool checked)
{
    settings->set("main/list_use_regex", checked);
}

void SettingsDialog::genRestrictionsList()
{
    QStringList l;

    for (int i=0; i < ui.restrictionsListView->count(); ++i)
        l << ui.restrictionsListView->item(i)->data(Qt::DisplayRole).toString();

    settings->restrictions = l;
}

void SettingsDialog::on_restrictionAddBtn_clicked()
{
    AddUrlDialog d;
    int r = d.exec();
    QString url = d.url();
    if ((r == QDialog::Accepted) && (! url.isEmpty()))
            ui.restrictionsListView->addItem(url);

    genRestrictionsList();
}

void SettingsDialog::on_restrictionDeleteBtn_clicked()
{
    ui.restrictionsListView->setUpdatesEnabled(false);
    QModelIndex idx = ui.restrictionsListView->selectionModel()->selectedIndexes().at(0);
    ui.restrictionsListView->model()->removeRow(idx.row());
    ui.restrictionsListView->setUpdatesEnabled(true);
    genRestrictionsList();
}

void SettingsDialog::on_restrictionImportBtn_clicked()
{
    QString filename = QFileDialog::getOpenFileName(this, "Import");
    if (! filename.isEmpty())
        restrictionsLoadFromFile(filename);
}

void SettingsDialog::on_restrictionExportBtn_clicked()
{
    QString filename = QFileDialog::getOpenFileName(this, "Export");
    if (! filename.isEmpty())
        restrictionsSaveToFile(filename);
}

/* display */

void SettingsDialog::on_hideMouseChk_toggled(bool checked)
{
    settings->set("slideshow/hide_pointer", checked);
}

void SettingsDialog::on_muteChk_toggled(bool checked)
{
    settings->set("slideshow/mute", checked);
}

void SettingsDialog::on_displayTimeSpin_valueChanged(int value)
{
    settings->set("slideshow/default_time", value);
}

void SettingsDialog::on_playlistBtnGrp_buttonClicked(QAbstractButton *btn)
{
    bool online = (btn == ui.updateOnlineRadio);
    bool local = (btn == ui.updateLocalRadio);
    bool b = (online||local);
    settings->set("slideshow/use_remote", b);
    settings->set("slideshow/use_local",local);
    ui.stackedWidget_7->setCurrentIndex(!b);
    ui.remoteEdit->setEnabled(local);
    ui.refreshPlaylistSpin->setEnabled(b);
    ui.playlistUsbUpdate->setEnabled(!b);
    ui.channelLabel->setVisible(online);
    if(b)loadChannels();
}

void SettingsDialog::on_playlistUsbUpdate_toggled(bool checked)
{
    settings->set("slidehow/do_local_update", checked);
}

void SettingsDialog::on_remoteEdit_textChanged(QString value)
{
    settings->set("slideshow/url", value);
}

void SettingsDialog::on_refreshPlaylistSpin_valueChanged(int value)
{
    settings->set("slideshow/refresh_after_minutes", value);
}

void SettingsDialog::on_localPlaylistDeleteBtn_clicked()
{
    m_localPlaylistView->delSelection();
    genPlaylistList();
}

void SettingsDialog::on_localPlaylistUpBtn_clicked()
{
    m_localPlaylistView->moveSelection(-1);
    genPlaylistList();
}

void SettingsDialog::on_localPlaylistDownBtn_clicked()
{
    m_localPlaylistView->moveSelection(1);
    genPlaylistList();
}

void SettingsDialog::on_localPlaylistAddBtn_clicked()
{
    AddPlaylistItemDialog d(settings->get("slideshow/default_time").toInt(), this);
    if (d.exec() != QDialog::Accepted)
        return;

    QString uri = d.text();
    uri = uri.trimmed();
    if (uri.isEmpty())
        return;

    if (uri.at(0) == '/')
        uri = QString("file://%1").arg(uri);
    int time = d.time();

    m_localPlaylistView->add(uri, time);

    genPlaylistList();
}

void SettingsDialog::on_localPlaylistCookiesBtn_clicked()
{
    QStringList urls;
    QStandardItemModel *m = static_cast<QStandardItemModel *>(m_localPlaylistView->model());

    for (int row=0; row < m->rowCount(); ++row)
    {
        QString url = m->item(row, 0)->data(Qt::DisplayRole).toString();
        if (url.startsWith("http://") || url.startsWith("https://"))
            urls << url;
    }

    CookieSaver cs(&urls, this);
    if (cs.exec() == QDialog::Accepted)
        settings->cookiesList = cs.allCookies();
}

void SettingsDialog::on_PlaylistSelectionChanged(
        const QItemSelection &selected,
        const QItemSelection &unselected)
{
    bool b = m_localPlaylistView->selectionModel()->hasSelection();
    ui.localPlaylistDeleteBtn->setEnabled(b);
    ui.localPlaylistUpBtn->setEnabled(b);
    ui.localPlaylistDownBtn->setEnabled(b);
}

/* network */
void SettingsDialog::on_proxyBtnGrp_buttonClicked(QAbstractButton *btn)
{
    int t;
    bool b;

    if (btn == ui.proxyAutoRadio)
    {
        t = 1;
        b = false;
    }
    else if (btn == ui.proxyManualRadio)
    {
        t = 2;
        b = true;
    }
    else
    {
        t = 0;
        b = false;
    }

    settings->set("network/proxy_type", t);
    ui.proxyUrlEdit->setEnabled(b);
    ui.proxyUserEdit->setEnabled(b);
    ui.proxyPasswordEdit->setEnabled(b);
}

void SettingsDialog::on_proxyUserEdit_textChanged(QString s)
{
    settings->set("network/proxy_user", s);
}

void SettingsDialog::on_proxyPasswordEdit_textChanged(QString s)
{
    settings->set("network/proxy_passwd", s);
}

void SettingsDialog::on_proxyUrlEdit_textChanged(QString s)
{
    settings->set("network/proxy_uri", s);
}

void SettingsDialog::on_connect_clicked()
{
    settings->set("slideshow/invoice",ui.invoice->text());
    settings->set("slideshow/email",ui.mail->text());
    loadChannels();
}

void SettingsDialog::on_listWidget_itemClicked(QListWidgetItem *item)
{
    QString url = settings->get("slideshow/use_local").toBool()?settings->get("slideshow/url").toString():"http://linutop.tv";
    settings->set("slideshow/pages",url+"/"+item->data(32).toString());
    settings->set("slideshow/channel",item->text());
    ui.channelLabel->setText(item->text());
}

void SettingsDialog::on_remoteChk_toggled(bool checked)
{
    settings->set("remote/enabled",checked);
    ui.directChk->setEnabled(checked);
    if(ui.remoteChk->isChecked()){
        QString ip = QNetworkInterface::allInterfaces().at(1).addressEntries().first().ip().toString();
        ip = ip.isEmpty()?"127.0.0.1":ip;
        ip += ":8080";
        ip = "http://" + ip;
        ui.remoteLabel->setText("<a href='"+ip+"'>"+ip+"</a>");
    }
    else{
        ui.remoteLabel->setText("");
    }
}

void SettingsDialog::on_directChk_toggled(bool checked)
{
    settings->set("slideshow/direct",checked);
}

void SettingsDialog::on_chooseTemplateBtn_clicked()
{
    newTemplate = QFileDialog::getOpenFileName(this,tr("Choose a template"),QString(),"(*.html *.htm)");
}

void SettingsDialog::on_cleanAfter_currentIndexChanged(int index)
{
    settings->set("slideshow/clean_after",ui.cleanAfter->itemData(index).toString());
}

void SettingsDialog::on_newTabUrl_textChanged(const QString &arg1)
{
    settings->set("main/new_tab_url",arg1);
}

void SettingsDialog::on_resizeChk_toggled(bool checked)
{
    settings->set("slideshow/resize",checked);
}
